package menu;

import javax.microedition.lcdui.*;
/**
* A Page is the basic element in a hierarchy of displayable
* pages.  This abstract class is extended to define
* specific behavior by specific types of pages.
* <p>If a subclass wishes to have a separate thread,
* it must implement the Runnable interface.  The separate
* thread is created and started by calling the 
* start() method of Page.
*
* <p>$Id: Page.java,v 1.4 2003/01/21 00:22:08 finson Exp $
*/
public abstract class Page {
	/**
	* Initialize a new Page object using the supplied parameters.
	* @param label short label for this page (as might be used in
	* a menu).  Must not be null.
	* @param title longer label for this page (as might be used in
	* a screen title).
	* @param icon small icon for this page (as might be used in
	* a toolbar or menu.
	*/
	public Page(String label,String title,Image icon) {
		if (label==null) throw new IllegalArgumentException("null Page label not allowed.");
		this.label = label;
		this.title = title;
		this.icon = icon;
	}
	/**
	* Return the Displayable object for this Page, completely filled in
	* and ready to display.
	* @return the appropriate Displayable object for this Page
	*/
	public abstract Displayable getDisplayable();
	/**
	* Process the user command and return an integer value telling
	* the PageMgr what to do next.  This method handles the details
	* of a command for a page, while the commandAction method in the
	* PageMgr handles the general responsibility of switching displays.
	* @param c a Command object identifying the command.
	* @param d the Displayable on which this event has occurred
	* @return a pageAction code, selected from the public fields defined
	* by PageMgr.
	*/
	public abstract int doCommandAction(Command c,Displayable d);
	/**
	* Create a separate thread and start it running.  The run method
	* of the concrete class that extends Page is the run method that
	* is used.  The concrete class must implement the Runnable
	* interface to let us know that the run() method exists.
	* @throws ClassCastException if the concrete class does not implement
	* the Runnable interface.
	* @throws IllegalStateException if there is a Page thread 
	* already running.  This means that if a thread hangs
	* on some long-term blocking call and the user cancels out of it and
	* tries the same page again before the thread actually exits, then the
	* midlet will die with this error unless the caller of start() catches
	* the exception.
	*/
	public synchronized void start() {
		if (!(this instanceof Runnable))
			throw new ClassCastException(this.getClass().getName()+" is not Runnable.");
		if (run) 
			throw new IllegalStateException("Page thread already running.");
		if (theThread != null)
			throw new IllegalStateException("Page thread still running.");

		run = true;
		theThread = new Thread((Runnable)this);
		theThread.start();
	}
	/**
	* Return the Page that this Page feels should be the next page
	* to be displayed.  This method is called by the PageMgr after
	* doCommandAction returns PageMgr.LINK as the pageAction value.
	* If the subclass can return this pageAction, then it must 
	* override this implementation of getLinkedPage with an 
	* appropriate implementation that does not throw an Exception.
	* @throws IllegalStateException this implementation always 
	* throws this exception
	* @return Page the next Page to be displayed
	*/
	public Page getLinkedPage() {
		throw new IllegalStateException(
		"getLinkedPage() not implemented in "+
		this.getClass().getName());
	}
	/**
	* Return Page label.
	* @return Page label
	*/
	public String getLabel() {
		return label;
	}
	/**
	* Return Page title.
	* @return Page title
	*/
	public String getTitle() {
		return title;
	}
	/**
	* Return Page icon.
	* @return Page icon
	*/
	public Image getIcon() {
		return icon;
	}
	/**
	* Return pointer to previous page visited.
	* @return previous Page
	*/
	public Page getPrevious() {
		return previous;
	}
	/**
	* Set pointer to previous page visited.
	* @param p previous Page
	*/
	public void setPrevious(Page p) {
		previous = p;
	}
	//----------------------------------------------------------------------
	
	/** short label for this page (as might be used in a menu) */
	private String label;
	/** longer label for this page (as might be used in a screen title) */
	private String title;
	/** small icon for this page (as might be used in a toolbar or menu) */
	private Image icon;
	
	/** pointer to the previously visited Page */
	private Page previous = null;
		
	/** 
	* Set to true while the dynamic content thread is running.
	* Set to false to indicate that the dynamic content thread should exit ASAP
	* and should definitely not touch anything like the screen.  Once this is
	* set to false, the dynamic content thread is no longer welcome in Dodge City.
	*/
	protected boolean run = false;
	/**
	* The Thread object representing the dynamic content thread.  The content 
	* thread sets this to null to acknowledge that it is exiting as requested.
	* This means that if the content thread does not exit gracefully and fails
	* to reset theThread, then the page will not be able to reload.
	*/
	protected Thread theThread = null;
}
